#! /bin/sh
 # This file (malloc_free.sh) was created by Ron Rechenmacher <ron@fnal.gov> on
 # Nov 15, 2016. "TERMS AND CONDITIONS" governing this file are in the README
 # or COPYING file. If you do not have such a file, one can be obtained by
 # contacting Ron or Fermi Lab in Batavia IL, 60510, phone: 630-840-3000.
 # $RCSfile: .emacs.gnu,v $
 # rev='$Revision: 1.30 $$Date: 2016/03/01 14:27:27 $'

delay_factor=60
#delay_factor=1
#delay_factor=2
iperf_node=
ssh_o='IdentityFile=/home/ron/.ssh/id_rsa'
opt_disk_write_only=--no-read  # for write only set to --no-read, for read/write set to NULL
disk_data_dir=/mnt/md0ssd/ron/tt
ganglia_disk_dev=md127
ganglia_node=
ganglia_cluster="mu2e DAQ"
do_display=

USAGE="\
   usage: `basename $0`
          `basename $0` 2>&1 | tee `basename $0`.out
examples: `basename $0` --iperf-node=mu2edaq06-data 2>&1 | tee `basename $0`.out
          `basename $0` --ganglia=mu2edaq01 --test-dir=/mnt/md0ssd/ron/tt 2>&1 | tee `basename $0`.out
--ganglia=
--iperf-node=
--test-dir=
--do-display   automatically display the key graphs
"
VUSAGE=""

eval env_opts=\${`basename $0 | sed 's/\.sh$//' | tr 'a-z-' 'A-Z_'`_OPTS-} # can be args too
eval "set -- $env_opts \"\$@\""
# Process script arguments and options
op1chr='rest=`expr "$op" : "[^-]\(.*\)"`; test -n "$rest" && set -- "-$rest" "$@"'
op1arg='rest=`expr "$op" : "[^-]\(.*\)"`; test -n "$rest" && set -- "$rest"  "$@"'
reqarg="$op1arg;"'test -z "${1+1}" &&echo opt -$op requires arg. &&echo "$USAGE" &&exit'
args= do_help= opt_v=0
while [ -n "${1-}" ];do
    if expr "x${1-}" : 'x-' >/dev/null;then
        op=`expr "x$1" : 'x-\(.*\)'`; shift   # done with $1
        leq=`expr "x$op" : 'x-[^=]*\(=\)'` lev=`expr "x$op" : 'x-[^=]*=\(.*\)'`
        test -n "$leq"&&eval "set -- \"\$lev\" \"\$@\""&&op=`expr "x$op" : 'x\([^=]*\)'`
        case "$op" in
        \?*|h*)      eval $op1chr; do_help=1;;
        v*)          eval $op1chr; opt_v=`expr $opt_v + 1`;;
        x*)          eval $op1chr; test $opt_v -ge 1 && set -xv || set -x;;
        -iperf-node) eval $reqarg; iperf_node=$1;                   shift;;
        -ganglia)    eval $reqarg; ganglia_node=$1;                 shift;;
        -test-dir)   eval $reqarg; disk_data_dir=$1;                shift;;
        esac
    else # allow mix of opts and arg (i.e. opts after args)
        aa=`echo "$1" | sed -e "s/'/'\"'\"'/g"` args="$args '$aa'"; shift
    fi
done
eval "set -- $args \"\$@\""; unset args aa
test $# -gt 2 && echo 'Too many arguments. $#='$# && do_help=1
test -n "${do_help-}" && { echo "$USAGE";test $opt_v -ge 1 && echo "$VUSAGE"; exit; }
set -u


# Run as root so script can drop_caches
test `whoami` = root || { echo 'Must be root (so script can drop_caches)'; exit; }

gevent() { description=$1
    wget -O/dev/null -q\
        "$ganglia_node/ganglia/api/events.php?action=add&start_time=now&summary=${description}&host_regex=`hostname -s`"\
 || echo "Error setting ganglia event: $description"
}

# ---------------------------------------------------------------------


if [ $delay_factor -eq 60 ];then
    echo "`date`: waiting for minute multiple of 5"
    min=`date +%M`
    until awk "BEGIN{m=$min%5;exit m==0?0:1}";do sleep 1; min=`date +%M`;done
    echo "`date`: proceeding, sending start event"
fi

gevent "start malloc_free test, drop, clear"
echo 1 >|/proc/sys/vm/drop_caches
# clear metrics
gmetric -n ave     -v 0.0 -t double -u us/MB   -g memory -D "ave us/MB" -T "ave us/MB"
gmetric -n min     -v 0.0 -t double -u us/MB   -g memory -D "min us/MB" -T "min us/MB"
gmetric -n max     -v 0.0 -t double -u us/MB   -g memory -D "max us/MB" -T "max us/MB"
gmetric -n threads -v 0   -t int32  -u threads -g memory -D "threads mallocing" -T "threads mallocing"
gmetric -n ave_MBperalloc -v 0.0 -t double -u MB/alloc -g memory -D "ave_MB/alloc" -T "ave MB/alloc"
if [ $delay_factor -eq 60 ];then
    while awk "BEGIN{m=$min%5;exit m==0?0:1}";do sleep 1; min=`date +%M`;done
    until awk "BEGIN{m=$min%5;exit m==0?0:1}";do sleep 1; min=`date +%M`;done
else
    sleep `expr $delay_factor \* 5`
fi


gevent "start malloc_free exe"
malloc_free --max=16m --min=10m --hi=500m --lo=100m --threads=8 -t3600 &
pids=$!
sleep `expr $delay_factor \* 5`


gevent "start disk writing"
disk_cache_me.sh $disk_data_dir -f --blksiz=128k $opt_disk_write_only &
pids="$pids $!"
# wait for free to go below 1 GB
sleep `expr $delay_factor \* 5`


if [ -n "$iperf_node" ];then
    gevent "start network read/write"
    iperf_2D.sh $iperf_node ${ssh_o+-o $ssh_o} -t 3600 &
    pids="$pids $!"
    sleep `expr $delay_factor \* 5`
fi


gevent "drop_caches again"
start=`date +%s`; end=`expr $delay_factor \* 5 + $start`
while [ `date +%s` -lt $end ];do
    echo 1 >|/proc/sys/vm/drop_caches
    sleep 5
done


gevent "kill everything, clear metrics"
echo "`date`: STOP/KILL ALL PROCESSES (malloc_free,disk_cache,iperf)"
for pp in $pids;do
    for sig in INT HUP TERM;do
        if kill -s 0 $pp 2>/dev/null;then
            echo `basename $0` killing $pp with $sig; kill -$sig $pp; sleep 1
        else
            break
        fi
    done
done
wait $pids
echo "`date`: ALL PROCESSES (malloc_free,disk_cache,iperf) COMPLETE"
gmetric -n ave     -v 0.0 -t double -u us/MB   -g memory -D "ave us/MB" -T "ave us/MB" 
gmetric -n min     -v 0.0 -t double -u us/MB   -g memory -D "min us/MB" -T "min us/MB" 
gmetric -n max     -v 0.0 -t double -u us/MB   -g memory -D "max us/MB" -T "max us/MB" 
gmetric -n threads -v 0   -t int32  -u threads -g memory -D "threads mallocing" -T "threads mallocing" 
gmetric -n ave_MBperalloc -v 0.0 -t double -u MB/alloc -g memory -D "ave_MB/alloc" -T "ave MB/alloc"
sleep `expr $delay_factor \* 2`


# get ganglia plots
size=large # xxlarge, xlarge, large, medium, small
event=   # '&event=hide&ts=0' or ''
period=30min  # i.e. 30min, hour
ganglia_cluster=`echo "$ganglia_cluster" | tr ' ' '+'`
curl --silent -o ave.png       "http://$ganglia_node/ganglia/graph.php?r=$period&z=$size&c=$ganglia_cluster&h=`hostname`&jr=&js=$event&m=ave&vl=us%2FMB&ti=ave+us%2FMB"
curl --silent -o max.png       "http://$ganglia_node/ganglia/graph.php?r=$period&z=$size&c=$ganglia_cluster&h=`hostname`&jr=&js=$event&m=max&vl=us%2FMB&ti=max+us%2FMB"
curl --silent -o min.png       "http://$ganglia_node/ganglia/graph.php?r=$period&z=$size&c=$ganglia_cluster&h=`hostname`&jr=&js=$event&m=min&vl=us%2FMB&ti=min+us%2FMB"
curl --silent -o threads.png   "http://$ganglia_node/ganglia/graph.php?r=$period&z=$size&c=$ganglia_cluster&h=`hostname`&jr=&js=$event&m=threads&vl=threads&ti=threads+mallocing"
curl --silent -o peralloc.png  "http://$ganglia_node/ganglia/graph.php?r=$period&z=$size&c=$ganglia_cluster&h=`hostname`&jr=&js=$event&m=ave_MBperalloc&vl=MB%2Falloc&ti=ave+MB%2Falloc"
curl --silent -o cached.png    "http://$ganglia_node/ganglia/graph.php?r=$period&z=$size&c=$ganglia_cluster&h=`hostname`&jr=&js=$event&m=mem_cached&vl=KB&ti=Cached+Memory"
curl --silent -o free.png      "http://$ganglia_node/ganglia/graph.php?r=$period&z=$size&c=$ganglia_cluster&h=`hostname`&jr=&js=$event&m=mem_free&vl=KB&ti=Free+Memory"
curl --silent -o dsk_read.png  "http://$ganglia_node/ganglia/graph.php?r=$period&z=$size&c=$ganglia_cluster&h=`hostname`&jr=&js=$event&m=diskstat_${ganglia_disk_dev}_read_bytes_per_sec&vl=bytes%2Fsec&ti=diskstat_${ganglia_disk_dev}_read_bytes_per_sec"
curl --silent -o dsk_write.png "http://$ganglia_node/ganglia/graph.php?r=$period&z=$size&c=$ganglia_cluster&h=`hostname`&jr=&js=$event&m=diskstat_${ganglia_disk_dev}_write_bytes_per_sec&vl=bytes%2Fsec&ti=diskstat_${ganglia_disk_dev}_write_bytes_per_sec"
curl --silent -o net_recv.png  "http://$ganglia_node/ganglia/graph.php?r=$period&z=$size&c=$ganglia_cluster&h=`hostname`&jr=&js=$event&m=bytes_in&vl=bytes%2Fsec&ti=Bytes+Received'"
curl --silent -o net_send.png  "http://$ganglia_node/ganglia/graph.php?r=$period&z=$size&c=$ganglia_cluster&h=`hostname`&jr=&js=$event&m=bytes_out&vl=bytes%2Fsec&ti=Bytes+Sent"
curl --silent -o pgpgin.png    "http://$ganglia_node/ganglia/graph.php?r=$period&z=$size&c=$ganglia_cluster&h=`hostname`&jr=&js=$event&m=vm_pgpgin&vl=ops%2Fsec&ti=Pages+in+from+disk"
curl --silent -o pgpgout.png   "http://$ganglia_node/ganglia/graph.php?r=$period&z=$size&c=$ganglia_cluster&h=`hostname`&jr=&js=$event&m=vm_pgpgout&vl=ops%2Fsec&ti=Pages+out+to+disk"

if [ -n "$do_display" ];then
    #for ff in ave max min threads cached dsk_read dsk_write net_recv net_send; do
    for ff in ave max min cached; do
        display $ff.png &
    done
fi

echo "`date`: END"
#kill 0
